########################## PS Core ##########################

# How to install Powershell Core LINK --> https://docs.microsoft.com/en-us/powershell/scripting/setup/installing-powershell-core-on-windows?view=powershell-6
# Powershell Core LINK (GitHub) --> https://github.com/PowerShell/PowerShell/

Start-Process 'https://docs.microsoft.com/en-us/powershell/scripting/setup/installing-powershell-core-on-windows?view=powershell-6'
Start-Process 'https://github.com/PowerShell/PowerShell/'

choco install powershell-core -y
# Install chocolatey --> https://chocolatey.org/docs/installation

# Copy-Paste popravek za PS Core
Install-Module PSReadLine -AllowPrerelease -AllowClobber -SkipPublisherCheck -Force

########################## PS gallery ##########################

Start-Process "https://www.powershellgallery.com/" # AWS,Cisco,Dell,...

Find-Module
Find-Script
Find-Module -Name *DNS* | ogv # Filter - brez Azure
Find-Script -Name *DNS*

Find-Module | ? Author -EQ "Microsoft Corporation" | ogv
Get-InstalledModule

Set-PSRepository -Name PSGallery -InstallationPolicy Trusted 

Start-Process https://www.powershellgallery.com/packages/PowerShellCookbook/1.3.6

$Namizje = "$ENV:USERPROFILE\Desktop"
Save-Module -Name PowerShellCookbook -Path $Namizje
ii "$Namizje\PowerShellCookbook" # PowerShellCookbook.psm1 --> Show-Object

Get-Alias ii

Install-Module -Name PowerShellCookbook -AllowClobber -Force
#Uninstall-Module

Get-Service -Name Spooler | Show-Object

########################## ConvertTo-FlatObject ##########################

Start-Process "http://gallery.technet.microsoft.com/"

# Poiscemo ConvertTo-FlatObject
# Shranimo

Get-Process | select -First 2 -Property * 
Get-Process | select -First 2 -Property * | Show-Object
Get-Process | select -First 2 -Property * | ConvertTo-FlatObject | clip

$FormatEnumerationLimit
$FormatEnumerationLimit=-1

########################## Parse WebPage ##########################

$uri = 'https://github.com/PowerShell/PowerShell/releases'
Invoke-WebRequest $uri

[Net.ServicePointManager]::SecurityProtocol
[Net.ServicePointManager]::SecurityProtocol = [Net.ServicePointManager]::SecurityProtocol -bor [Net.SecurityProtocolType]::Tls12

# bor 	Bitwise OR
# https://docs.microsoft.com/en-us/powershell/module/microsoft.powershell.core/about/about_arithmetic_operators?view=powershell-5.1

$x = Invoke-WebRequest $uri
$x.ToString() | sls download_count
$x | Show-Object
$x | ConvertTo-FlatObject | clip
$x.AllElements | ConvertTo-FlatObject | clip

get-alias irm,iwr

# https://en.wikipedia.org/wiki/Representational_state_transfer
# https://developer.github.com/v3/

$y = Invoke-RestMethod "api.github.com/repos/PowerShell/PowerShell/releases"
Show-Object $y
$y | ConvertTo-FlatObject | clip

$y.assets | sort -Descending down* | ft Name,d* -AutoSize

########################## Po korakih ##########################

gps -ov step1 | ? name -like *ss -ov step2 | group name -ov step3

########################## Convert-String ##########################

"Mu Han", "Jim Hance", "David Ahs", "Kim Akers" | Convert-String -Example "Joze Novak=Novak, J."

$a = gps -Name svchost | select processname, id | ConvertTo-Csv -NoTypeInformation | select -Last 13
$a | Convert-String -Example '"svchost", "219"=219, s.'

########################## Shrani ISE ##########################

$psISE.PowerShellTabs.Files | Where-Object {$_.IsUntitled} | ForEach-Object { $_.Editor.Text } | clip.exe

########################## Spremeni lastnosti ##########################

$File = Get-Item 'E:\Porocilo.txt'
$Date = Get-Date

$File.creationtime
$File.lastwritetime
$File.lastaccesstime

$File.creationtime=$Date
$File.lastwritetime=$Date
$File.lastaccesstime=$Date

########################## POPUP ##########################

[void][System.Reflection.Assembly]::LoadWithPartialName('Microsoft.VisualBasic')
$name = [Microsoft.VisualBasic.Interaction]::InputBox("Enter your name", "Name", "$env:username")
$name

########################## Try, Catch, Finally ##########################

# https://docs.microsoft.com/en-us/powershell/module/microsoft.powershell.core/about/about_try_catch_finally?view=powershell-5.1

Try {
    iwr KODA
}
Catch [System.OutOfMemoryException] {
    Restart-Computer -Force
}
Catch [System.Management.Automation.CommandNotFoundException],[System.IO.IOException] {
    "Prvi catch"
}
Catch {
    "Drugi catch"
}
Finally {
    $Time = Get-Date
    "This script tried to run at $Time"    # Lahko Out-File v Log
}

########################## OGV & PassThru ##########################

$logs = get-eventlog -list | select -expand log | out-gridview -Title "Select on or more event logs" -PassThru
$count = 5,15,20,25 | out-gridview -title "How many entries?" -OutputMode Single
$Object = @()
foreach ($log in $logs) {
    $Object += Get-Eventlog $log -Newest $count |
        select @{Name="Log";Expression={$log}},TimeGenerated,EntryType,Source,EventID,Message
}

$Object | Out-GridView -Title "Event Log Report"

########################## Measure primer ##########################

Write-Host "Array with += operators:"
$array = @()

Measure-Command {
	for ($i = 1; $i -le 15000; $i++) { $array += "Array Element $i" }
}

############ List ############

Write-Host "List:"
$array = @()
$list = New-Object System.Collections.Generic.List[System.String]

Measure-Command {
	for ($i = 1; $i -le 15000; $i++) { $list.Add("Array Element $i") }
    $array = $list.ToArray()
}

########################## Measure & Filter ##########################

$Time1 = Measure-Command {Get-ADUser -Filter "GivenName -like 'Marko'"}
$Time2 = Measure-Command {Get-ADUser -Filter * | Where-Object "GivenName -like 'Marko'"}

$Time2.TotalMilliseconds / $Time1.TotalMilliseconds

# Ali zgornje meritve vedno veljajo ???